#include <iostream>
#include <SDL/SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include "Texture.h"
#include "CelScene.h"

#include "../models/spikeThingie2.h"

using std::cout;
using std::endl;

CelScene::CelScene(SDL_Surface *sdlsurf)
{
    surface = sdlsurf;

    ting = new Texture("data/lol.png");

    //init();

    // initialize member variables
    outlineSmooth = true;
    outlineWidth = 3.0;
}

CelScene::~CelScene()
{
    cout << framesDrawn << " frames draw\tAverage fps: " <<
        (float)framesDrawn/((float)getTime()/1000.0) << endl;

    glDeleteTextures(1, &shaderTexture[0]);        // Delete The Shader Texture

    glDisable(GL_TEXTURE_1D);
    glLineWidth(1.0);

    delete ting;
}

void CelScene::render()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();

    // background
    if (getTime() > 2000 && getTime() < 8000) {
        set640x480();
        glEnable(GL_TEXTURE_2D);
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        ting->bind();
        glDisable(GL_DEPTH_TEST);
        glColor4f(1.0, 1.0, 1.0, 1.0);
        if (getTime() < 3000)
            glColor4f(1.0, 1.0, 1.0, ((float)getTime()-2000)/1000.0);
        else if (getTime() > 7000)
            glColor4f(1.0, 1.0, 1.0, 1.0-((float)getTime()-7000)/1000.0);
        glBegin(GL_QUADS);
        {
            glTexCoord2i(0, 0); glVertex2i(2, 511);
            glTexCoord2i(1, 0); glVertex2i(514, 511);
            glTexCoord2i(1, 1); glVertex2i(514, 0);
            glTexCoord2i(0, 1); glVertex2i(2, 0);
        }
        glEnd();
        glEnable(GL_DEPTH_TEST);
        glDisable(GL_TEXTURE_2D);
        glDisable(GL_BLEND);
        unsetOrtho();
    }

    float tmpShade;

    float tmpMatrix[16];
    Vector3d tmpVector, tmpNormal;

    if (outlineSmooth)
        glEnable(GL_LINE_SMOOTH);
    else
        glDisable(GL_LINE_SMOOTH);

    glTranslatef(0.0, 0.0, -2.0);
    glRotatef(getTime()/20.0, 0.1, 0.2, 0.3);

    glGetFloatv(GL_MODELVIEW_MATRIX, tmpMatrix);

    glBindTexture(GL_TEXTURE_1D, shaderTexture[0]);

    glColor3f(0.6, 0.9, 0.6);

    glBegin(GL_TRIANGLES);
    for (int i = 0; i < spikeThingie2_numberFaces; i++) {
        for (int j = 0; j < 3; j++) {
            tmpNormal.x = spikeThingie2_normals[spikeThingie2_faces[i][j]][0];
            tmpNormal.y = spikeThingie2_normals[spikeThingie2_faces[i][j]][1];
            tmpNormal.z = spikeThingie2_normals[spikeThingie2_faces[i][j]][2];

            tmpVector.rotate(tmpMatrix, tmpNormal);
            tmpVector.normalize();

            tmpShade = tmpVector.dotProduct(lightAngle);

            if (tmpShade < 0.0)
                tmpShade = 0.0;

            glTexCoord1f(tmpShade);
            glVertex3fv(spikeThingie2_vertices[spikeThingie2_faces[i][j]]);
        }
    }   
    glEnd();

    // draw outline
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

    glPolygonMode(GL_BACK, GL_LINE);
    glLineWidth(outlineWidth);

    glCullFace(GL_FRONT);

    glDepthFunc(GL_LEQUAL);

    glColor3f(0.0, 0.0, 0.0);
    glBegin(GL_TRIANGLES);
    { 
        for (int i = 0; i < spikeThingie2_numberFaces; i++) {
            glVertex3fv(spikeThingie2_vertices[spikeThingie2_faces[i][0]]);
            glVertex3fv(spikeThingie2_vertices[spikeThingie2_faces[i][1]]);
            glVertex3fv(spikeThingie2_vertices[spikeThingie2_faces[i][2]]);
        }   
    }
    glEnd();

    glCullFace(GL_BACK);

    // draw transperant wireframe
    glPolygonMode(GL_FRONT, GL_LINE);
    glLineWidth(1.0);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

    glColor4f(0.0, 0.0, 0.0, 0.5);
    glScalef(1.01, 1.01, 1.01);
    glBegin(GL_TRIANGLES);
    { 
        for(int i = 0; i < spikeThingie2_numberFaces; i++) {
            glVertex3fv(spikeThingie2_vertices[spikeThingie2_faces[i][0]]);
            glVertex3fv(spikeThingie2_vertices[spikeThingie2_faces[i][1]]);
            glVertex3fv(spikeThingie2_vertices[spikeThingie2_faces[i][2]]);
        }   
    }
    glEnd();
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

    glDepthFunc(GL_LESS);
    glPolygonMode(GL_BACK, GL_FILL);
    glDisable(GL_BLEND);

    // FIXME
    // fade out
    if (getTime() > 14000) {
        setOrtho();
        glDisable(GL_DEPTH_TEST);
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glColor4f(0.0, 0.0, 0.0, ((float)(getTime()-14000))/1000.0);
        glBegin(GL_QUADS);
        {
            glVertex2i(-1,-1);
            glVertex2i( 1,-1);
            glVertex2i( 1, 1);
            glVertex2i(-1, 1);
        }
        glEnd();
        glEnable(GL_DEPTH_TEST);
        glDisable(GL_BLEND);
        unsetOrtho();
    }

    // FIXME
    // fade in
    if (getTime() < 1000) {
        setOrtho();
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glDisable(GL_DEPTH_TEST);
        glColor4f(0.0, 0.0, 0.0, 1.0-(float)getTime()/1000.0);
        glBegin(GL_QUADS);
        {
            glVertex2i(-1,-1);
            glVertex2i( 1,-1);
            glVertex2i( 1, 1);
            glVertex2i(-1, 1);
        }
        glEnd();
        glEnable(GL_DEPTH_TEST);
        glDisable(GL_BLEND);
        unsetOrtho();
    }

    SDL_GL_SwapBuffers();
}

void CelScene::init()
{
    startTime = SDL_GetTicks();
    framesDrawn = 0;

    // OpenGL stuff
    glClearColor(0.7, 0.7, 0.7, 0.0);
    glViewport(0, 0, 640, 480);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0, 640.0/480.0, 1.0, 100.0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glShadeModel(GL_SMOOTH);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

    glClearDepth(1.0);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LESS);

    glDisable(GL_LINE_SMOOTH);
    glEnable(GL_CULL_FACE);
    glDisable(GL_LIGHTING);
    glEnable(GL_TEXTURE_1D);

    for (int i = 0; i < 32; i++) {
        // FIXME: tweaking needed :-)
        if (i <= 4)
            shaderData[i][0] = shaderData[i][1] = shaderData[i][2] = 0.2;
        else if (i <= 14)
            shaderData[i][0] = shaderData[i][1] = shaderData[i][2] = 0.5;
        else
            shaderData[i][0] = shaderData[i][1] = shaderData[i][2] = 1.0;
    }

    glGenTextures(1, &shaderTexture[0]);
    glBindTexture(GL_TEXTURE_1D, shaderTexture[0]);

    glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexImage1D(GL_TEXTURE_1D, 0, GL_RGB, 32, 0, GL_RGB , GL_FLOAT, shaderData);

    lightAngle.x = 0.0;
    lightAngle.y = 0.0;
    lightAngle.z = 1.0;

    lightAngle.normalize();

    glViewport(0, 0, 640, 480);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0, 640.0/480.0, 0.1, 100.0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    // return 0;
}

