#include <iostream>
#include <SDL/SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include "Vector3d.h"
#include "CylinderScene.h"

using std::cout;
using std::endl;

CylinderScene::CylinderScene(SDL_Surface *sdlsurf)
{
    // initialize all 30 circles
    for (int i = 0; i < 30; i++) {
        circles.push_back(new Circle(i));

        rotation[i] = 0.0;
    }

    init();

    glEnable(GL_DEPTH_TEST);

    //glEnable(GL_NORMALIZE);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST); // Really Nice Perspective Calculations
    glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
    glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
    glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
}

CylinderScene::~CylinderScene()
{
    cout << framesDrawn << " frames draw\tAverage fps: " <<
        (float)framesDrawn/((float)getTime()/1000.0) << endl;
    glDisable(GL_DEPTH_TEST);
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
}

void CylinderScene::render()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();

    glTranslatef(0.0, 0.0,-40.0);

    glColor3f(1.0, 1.0, 1.0);

//    glEnable(GL_LIGHTING);
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

    // draw all 29 (not 30!) quad-strips
    for (int i = 0; i < 29; i++) {
        glBegin(GL_TRIANGLE_STRIP);
        for (int j = 0; j < 32; j++) {
            glVertex3f(
                    circles[i]->points[j].x,
                    circles[i]->points[j].y,
                    circles[i]->points[j].z
                    );
            glVertex3f(
                    circles[i+1]->points[j].x,
                    circles[i+1]->points[j].y,
                    circles[i+1]->points[j].z
                    );
        }

        // close last gap
        glVertex3f(
                circles[i]->points[0].x,
                circles[i]->points[0].y,
                circles[i]->points[0].z
                );
        glVertex3f(
                circles[i+1]->points[0].x,
                circles[i+1]->points[0].y,
                circles[i+1]->points[0].z
                );

    }
    glEnd();

    for (int i = 0; i < 30; i++)
        rotateCircle(circles[i], rotation[i]);

    SDL_GL_SwapBuffers();

    updateRotations();
}

void CylinderScene::rotateCircle(Circle* circle, float rot)
{
    float angle;
    float PI = 3.141592654;

    for (int j = 0; j < 32; j++) {
        angle = (float)((2.0*PI)/32.0*j);

        circle->points[j].x = circle->radiuses[j]*sin(angle+rot);
        //points[j].y = (float)i-14.5; // z-values from + to -14.5
        circle->points[j].z = circle->radiuses[j]*cos(angle+rot);      
    }
}

void CylinderScene::updateRotations()
{
    rotation[0] = (float)getTime()/1000.0;

    for (int i = 1; i < 30; i++) {
        rotation[i] = rotation[i-1]*1.03;
    }
}
