
#include <SDL/SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include "Vector3d.h"
#include "Heightfield.h"

#include "../models/spikeThingie.h"

using std::cout;
using std::endl;

Heightfield::Heightfield(SDL_Surface *sdlsurf)
{
    surface = sdlsurf;

    bump = 17350;

    //init();

    genmap();

    dude = new Texture("data/tufs_dude.png");
    ting = new Texture("data/lol2.png");
    smegma = new Texture("data/smegma.png");
    wemade = new Texture("data/skaptav.png");


    mc = 0;
    mcounter = 0;
    rot = 0;
    ticks = 0;

    lightAngle.x = 0.0;
    lightAngle.y = 0.0;
    lightAngle.z = 1.0;

    lightAngle.normalize();

    outlineSmooth = true;
    outlineWidth = 3.0;
}

Heightfield::~Heightfield()
{
    deinit();
    cout << framesDrawn << " frames draw\tAverage fps: " <<
        (float)framesDrawn/((float)getTime()/1000.0) << endl;

    delete wemade;
    delete ting;
    delete smegma;
    delete dude;
}

void Heightfield::render()
{
    glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
    glLoadIdentity();

    // bilde
    if (getTime() < 10000)
    {
        glDisable(GL_DEPTH_TEST);
        dude->bind();
        glColor4f(1.0, 1.0, 1.0, 1.0);
        // fade out
        if (getTime() >= 7000)
            glColor4f(1.0, 1.0, 1.0, 1.0-(float)(getTime()-7000)/3000.0);

        set640x480();
        glBegin(GL_QUADS);
        {
            glTexCoord2i(0, 0); glVertex2i(63, 495);
            glTexCoord2i(1, 0); glVertex2i(575, 495);
            glTexCoord2i(1, 1); glVertex2i(575, -16);
            glTexCoord2i(0, 1); glVertex2i(63, -16);
        }
        glEnd();
        unsetOrtho();
        glEnable(GL_DEPTH_TEST);
        glBindTexture(GL_TEXTURE_2D, GL_NONE);
    }
    else {
        glColor4f(1.0, 1.0, 1.0, 1.0);
        if (getTime() < 12000)
            glColor4f(1.0, 1.0, 1.0, (float)(getTime()-10000)/3000.0);

        glDisable(GL_DEPTH_TEST);
        ting->bind();
        set640x480();
        glBegin(GL_QUADS);
        {
            glTexCoord2i(0, 0); glVertex2i(127, 495);
            glTexCoord2i(1, 0); glVertex2i(639, 495);
            glTexCoord2i(1, 1); glVertex2i(639, -16);
            glTexCoord2i(0, 1); glVertex2i(127, -16);
        }
        glEnd();
        unsetOrtho();
        glBindTexture(GL_TEXTURE_2D, GL_NONE);
        glEnable(GL_DEPTH_TEST);
    }

    // background
    if (getTime() < 10000) {
        set640x480();
        smegma->bind();
        glDisable(GL_DEPTH_TEST);
        glColor4f(1.0, 1.0, 1.0, 1.0);
        if (getTime() > 9000)
            glColor4f(1.0, 1.0, 1.0, 1.0-((float)getTime()-9000)/1000.0);
        glBegin(GL_QUADS);
        {
            glTexCoord2i(0, 0); glVertex2i(383, 255);
            glTexCoord2i(1, 0); glVertex2i(639, 255);
            glTexCoord2i(1, 1); glVertex2i(639, 0);
            glTexCoord2i(0, 1); glVertex2i(383, 0);
        }
        glEnd();
        glEnable(GL_DEPTH_TEST);
        glBindTexture(GL_TEXTURE_2D, GL_NONE);
        unsetOrtho();
    }

    // background
    if (getTime() > 2000 && getTime() < 8000) {
        set640x480();
        wemade->bind();
        glDisable(GL_DEPTH_TEST);
        glColor4f(1.0, 1.0, 1.0, 1.0);
        if (getTime() < 3000)
            glColor4f(1.0, 1.0, 1.0, ((float)getTime()-2000)/1000.0);
        else if (getTime() > 7000)
            glColor4f(1.0, 1.0, 1.0, 1.0-((float)getTime()-7000)/1000.0);
        glBegin(GL_QUADS);
        {
            glTexCoord2i(0, 0); glVertex2i(2, 495);
            glTexCoord2i(1, 0); glVertex2i(258, 495);
            glTexCoord2i(1, 1); glVertex2i(258, -26);
            glTexCoord2i(0, 1); glVertex2i(2, -16);
        }
        glEnd();
        glEnable(GL_DEPTH_TEST);
        glBindTexture(GL_TEXTURE_2D, GL_NONE);
        unsetOrtho();
    }

    {
        rot += 0.01;
        //if(mcounter == 50)
        //{
        // printf("\n**************\nmcounter = %d\n", mcounter);
        //      ticks -= ticks / 50;
        mc = getTime()/75;
        mcounter = 0;
        if(mc < 440)
        {
            mc++;
        }
        else
            mc %= 440;
        //}
        //else
        //    mcounter++;

        ticks = getTime();
    }
    //float z, x, y;

    //  glRotatef(rot, 1, 1, 1);

    /* render ___ map */
    glTranslatef(0, -1.5, -4);
    rendermap(mc, mc+50, 10, 50, -20, 10, -2, 2);

    glLoadIdentity();

    if (getTime() >= 17500) {
        if (getTime()-bump > 900) {
            if (getTime()-bump < 1000)
                glTranslatef(0.0, 0.0, 1.0-(float)(getTime()-bump)/1000.0);//, 0.0);
            else
                glTranslatef(0.0, 0.0, 1.0-(float)(getTime()-bump)/1000.0);//, 0.0);
        }
        if (getTime()-bump > 1100) {
            bump = getTime();
        }
    }

    if (getTime() >= 10000) {
        glColor4f(0.0, 0.0, 1.0, 1.0);
        if (getTime() < 12000) {
            glColor4f(0.0, 0.0, 1.0, 0.0);
            glTranslatef(2.0-(getTime()-10000)/1000.0, 0.0, 0.0);
        }
        else if (getTime() <= 14000)
            glColor4f(0.0, 0.0, 1.0, (float)(getTime()-12000)/2000.0);

        glTranslatef(0.0, 0.0, -2.0);
        renderCelPr0n();
    }

    // FIXME
    // fade out
    if (getTime() > 34000) {
        setOrtho();
        glDisable(GL_DEPTH_TEST);
        glColor4f(0.0, 0.0, 0.0, ((float)(getTime()-34000))/1000.0);
        glBegin(GL_QUADS);
        {
            glVertex2i(-1,-1);
            glVertex2i( 1,-1);
            glVertex2i( 1, 1);
            glVertex2i(-1, 1);
        }
        glEnd();
        glEnable(GL_DEPTH_TEST);
        unsetOrtho();
    }

    // FIXME
    // fade in
    if (getTime() < 1000) {
        setOrtho();
        glDisable(GL_DEPTH_TEST);
        glColor4f(0.0, 0.0, 0.0, 1.0-(float)getTime()/1000.0);
        glBegin(GL_QUADS);
        {
            glVertex2i(-1,-1);
            glVertex2i( 1,-1);
            glVertex2i( 1, 1);
            glVertex2i(-1, 1);
        }
        glEnd();
        glEnable(GL_DEPTH_TEST);
        unsetOrtho();
    }

    SDL_GL_SwapBuffers();
}

void Heightfield::init()
{
    glPushAttrib(GL_ALL_ATTRIB_BITS);

    startTime = SDL_GetTicks();
    framesDrawn = 0;

    // cel-stuff
    for (int i = 0; i < 32; i++) {
        // FIXME: tweaking needed :-)
        if (i <= 4)
            shaderData[i][0] = shaderData[i][1] = shaderData[i][2] = 0.2;
        else if (i <= 14)
            shaderData[i][0] = shaderData[i][1] = shaderData[i][2] = 0.5;
        else
            shaderData[i][0] = shaderData[i][1] = shaderData[i][2] = 1.0;
    }

    glGenTextures(1, &shaderTexture[0]);
    glBindTexture(GL_TEXTURE_1D, shaderTexture[0]);

    glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexImage1D(GL_TEXTURE_1D, 0, GL_RGB, 32, 0, GL_RGB , GL_FLOAT, shaderData);

    //// OpenGL stuff
    //glClearColor(0.7, 0.7, 0.7, 0.0);
    //glViewport(0, 0, 640, 480);
    //glMatrixMode(GL_PROJECTION);
    //glLoadIdentity();
    //gluPerspective(45.0, 640.0/480.0, 1.0, 100.0);

    //glMatrixMode(GL_MODELVIEW);
    //glLoadIdentity();

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0, 640.0/480.0, 1.0, 100.0);
    glMatrixMode(GL_MODELVIEW);
    //    glClearColor(0.3, 0.2, 0, 0);
    glClearColor(0.7, 0.7, 0.7, 0.0);
    //    glEnable(GL_DEPTH_TEST);
    glEnable(GL_TEXTURE_2D);

    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    // cel-stuff
    glShadeModel(GL_SMOOTH);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

    glClearDepth(1.0);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LESS);

    glDisable(GL_LINE_SMOOTH);
    glEnable(GL_CULL_FACE);
    glDisable(GL_LIGHTING);
    glEnable(GL_TEXTURE_1D);
}

float Heightfield::noise(float x, float y)
{

    int n = (int)(x + y * 57);
    n = (n << 13) ^ n;

    return (float)(1.0 - ((n * (n * n * 15731 + 789221) +
                    1376312589) & 0x7FFFFFFF) / 1073741824.0);
}

void Heightfield::rendermap(int start, int stopp, int px, int pz, int zmin, int zmax, int xmin, int xmax)
{

    /* indekser i mappen. */
    int i;
    /* skjemkoordinater. */
    float x, z;
    float blend = 0.5;

    if (getTime() > 16000)
        blend = 0.5-(float)(getTime()-16000)/4000.0;

    x = 2.0;

    /* start p indeks (start) i mappen, fortsett til indeks (stopp) nes.
       start koordinattellingen p (zmin) og fortsett til (zmax) nes. */

    float xd = (float)(xmax - xmin) / px;
    float zd = (float)(zmax - zmin) / pz;

    glPushAttrib(GL_ALL_ATTRIB_BITS);
    glRotatef(45, 0, 1, 0);
    //printf("%f\n", rot);
    for(i = start, z = zmin; i < stopp && z < zmax; i++, z += zd)
    {
        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
        //glEnable(GL_BLEND);
        //if(blend > 1)
        //    blend = 0;
        //else
        //    blend += 0.1;
        glColor4f(0.0, 0.5, 0.6, blend);
        glBegin(GL_QUADS);
        {
            glVertex3f(x,    map[i*10]  , z);
            glVertex3f(x+xd, map[i*10]  , z);
            glVertex3f(x+xd, map[i+1*10], z);
            glVertex3f(x,    map[i+1*10], z);
        }
        glEnd();
        //glDisable(GL_BLEND);

        glColor4f(0.0, 0.0, 0.7, blend);
        glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
        glBegin(GL_QUADS);
        {
            glVertex3f(x,    map[i*10]  , z);
            glVertex3f(x+xd, map[i*10]  , z);
            glVertex3f(x+xd, map[i+1*10], z);
            glVertex3f(x,    map[i+1*10], z);
        }
        glEnd();
    }


    glPopAttrib();
}

void Heightfield::genmap()
{

    int i; //, j;

    int lmao = 242; // getpid();

    for(i = 0; i < 5000; i++)
        map[i] = noise(i, lmao+i);

}

void Heightfield::renderCelPr0n()
{
    float tmpShade;

    float tmpMatrix[16];
    Vector3d tmpVector, tmpNormal;

    if (outlineSmooth)
        glEnable(GL_LINE_SMOOTH);
    else
        glDisable(GL_LINE_SMOOTH);

    glRotatef(getTime()/20.0, 0.1, 0.2, 0.3);

    glGetFloatv(GL_MODELVIEW_MATRIX, tmpMatrix);

    // Cel-Shading Code
    glBindTexture(GL_TEXTURE_1D, shaderTexture[0]);

    glBegin(GL_TRIANGLES);
    for (int i = 0; i < spikeThingie_numberFaces; i++) {
        for (int j = 0; j < 3; j++) {
            tmpNormal.x = spikeThingie_normals[spikeThingie_faces[i][j]][0];
            tmpNormal.y = spikeThingie_normals[spikeThingie_faces[i][j]][1];
            tmpNormal.z = spikeThingie_normals[spikeThingie_faces[i][j]][2];

            tmpVector.rotate(tmpMatrix, tmpNormal);
            tmpVector.normalize();

            tmpShade = tmpVector.dotProduct(lightAngle);

            if (tmpShade < 0.0)
                tmpShade = 0.0;

            glTexCoord1f(tmpShade);
            glVertex3fv(spikeThingie_vertices[spikeThingie_faces[i][j]]);
        }
    }   
    glEnd();

    // draw outline
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

    glPolygonMode(GL_BACK, GL_LINE);
    glLineWidth(outlineWidth);

    glCullFace(GL_FRONT);

    glDepthFunc(GL_LEQUAL);


    glBegin(GL_TRIANGLES);
    { 
        for (int i = 0; i < spikeThingie_numberFaces; i++) {
            glVertex3fv(spikeThingie_vertices[spikeThingie_faces[i][0]]);
            glVertex3fv(spikeThingie_vertices[spikeThingie_faces[i][1]]);
            glVertex3fv(spikeThingie_vertices[spikeThingie_faces[i][2]]);
        }   
    }
    glEnd();

    glCullFace(GL_BACK);

    // draw transperant wireframe
    glPolygonMode(GL_FRONT, GL_LINE);
    glLineWidth(1.0);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

    glColor4f(0.0, 0.0, 0.0, 1.0);
    if (getTime() <= 12000) {
        glColor4f(0.0, 0.0, 0.0, (float)(getTime()-10000)/2000.0);
    }
    glScalef(1.01, 1.01, 1.01);
    glBegin(GL_TRIANGLES);
    { 
        for(int i = 0; i < spikeThingie_numberFaces; i++) {
            glVertex3fv(spikeThingie_vertices[spikeThingie_faces[i][0]]);
            glVertex3fv(spikeThingie_vertices[spikeThingie_faces[i][1]]);
            glVertex3fv(spikeThingie_vertices[spikeThingie_faces[i][2]]);
        }   
    }
    glEnd();
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

    glDepthFunc(GL_LESS);
    glPolygonMode(GL_BACK, GL_FILL);
    //glDisable(GL_BLEND);

//    SDL_GL_SwapBuffers();

    glBindTexture(GL_TEXTURE_1D, GL_NONE);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}
