#include <iostream>
#include <GL/gl.h>
#include <GL/glu.h>
#include "Scene.h"

Scene::Scene()
{
    done = false;
}

Scene::~Scene()
{
}

int Scene::getTime()
{
    return SDL_GetTicks() - startTime;
}

void Scene::init()
{
    startTime = SDL_GetTicks();
    framesDrawn = 0;

    // OpenGL stuff
    glClearColor(0.0, 0.0, 0.0, 0.0);
    glViewport(0, 0, 640, 480);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0, 640.0/480.0, 1.0, 100.0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glShadeModel(GL_SMOOTH);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
}

void Scene::deinit()
{
    glPopAttrib();
}

int Scene::run(int milliseconds)
{
    // generic mainloop, run until esc is pressed or the window is closed
    while (true) {
        while(SDL_PollEvent(&event)) {
            switch(event.type) {
                case SDL_KEYDOWN:
                    switch(event.key.keysym.sym) {
                        case SDLK_ESCAPE:
                            return 1;
                            break;
                        //case 'f':
                        //    SDL_WM_ToggleFullScreen(surface);
                        //    break;
                        default:
                            break;
                    }
                    break;
                case SDL_QUIT:
                    done = getTime();
                    break;
                default:
                    break;
            }
        }
        render();
        framesDrawn++;

        if (done > 0 || getTime() >= milliseconds) {
            // could fade out or something here, but let's just quit and let
            // derived classes handle things as they want to
            return 0;
        }
    }

    return 0;
}

void Scene::render()
{
    // empty, derived classes will always have their own render method
}

void Scene::setOrtho()
{
    // Set 2D matrices
    glMatrixMode(GL_PROJECTION);  
    glPushMatrix();            
    glLoadIdentity();
    glOrtho(-1, 1, -1, 1, 1, -1);

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
}

void Scene::set640x480()
{
    // Set 2D matrices
    glMatrixMode(GL_PROJECTION);  
    glPushMatrix();            
    glLoadIdentity();
    glOrtho(0, 640, 480, 0, 1, -1);

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
}

void Scene::unsetOrtho()
{
    // Restore 3D matrices
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
}

