// 3d vector functions

#include <math.h>

#include "Vector3d.h"

Vector3d Vector3d::operator+(Vector3d op2)
{
    Vector3d temp;

    temp.x = x + op2.x;
    temp.y = y + op2.y;
    temp.z = z + op2.z;

    return temp;
}

Vector3d Vector3d::operator-(Vector3d op2)
{
    Vector3d temp;

    temp.x = x - op2.x;
    temp.y = y - op2.y;
    temp.z = z - op2.z;

    return temp;
}

Vector3d Vector3d::operator*(float op2)
{
    Vector3d temp;

    temp.x = x * op2;
    temp.y = y * op2;
    temp.z = z * op2;

    return temp;
}

float Vector3d::magnitude()
{
    return sqrt((x*x) + (y*y) + (z*z));
}

float Vector3d::dotProduct(Vector3d op2)
{
    return((this->x * op2.x) + (this->y * op2.y) + (this->z * op2.z));
}

Vector3d Vector3d::crossProduct(const Vector3d op2)
{
    Vector3d temp;

    temp.x = (this->y * op2.z) - (this->z * op2.y);
    temp.y = (this->z * op2.x) - (this->x * op2.z);
    temp.z = (this->x * op2.y) - (this->y * op2.x);

    return temp;
}

// make vector unit lenght
void Vector3d::normalize()
{
    Vector3d temp(x, y, z);

    if (magnitude() > 0) {
        temp = temp*(1/magnitude());
//        std::cerr << "Tried to normalize 0-vector\n";
    }
    else
        temp.x = temp.y = temp.z = 0.0;

    x = temp.x;
    y = temp.y;
    z = temp.z;
}

// rotate vector using a (float[16])-matrix
void Vector3d::rotate(float* mat, Vector3d op2)
{
    this->x = (mat[0] * op2.x) + (mat[4] * op2.y) + (mat[8]  * op2.z);
    this->y = (mat[1] * op2.x) + (mat[5] * op2.y) + (mat[9]  * op2.z);
    this->z = (mat[2] * op2.x) + (mat[6] * op2.y) + (mat[10] * op2.z);
}

